/******************************************************
                  DirectShow .NET
		      netmaster@swissonline.ch
*******************************************************/
//					DsUtils
// DirectShow utility classes, partial from the SDK Common sources

using System;
using System.Collections;
using System.IO;
using System.Text;
using System.Runtime.InteropServices;
using System.Diagnostics;

namespace DShowNET
{

	[ComVisible(false)]
	public class DsUtils
	{

		public static bool IsCorrectDirectXVersion()
		{
			return File.Exists( Path.Combine( Environment.SystemDirectory, @"dpnhpast.dll" ) );
		}


		public static bool ShowCapPinDialog( ICaptureGraphBuilder2 bld, IBaseFilter flt, IntPtr hwnd )
		{
			int hr;
			object comObj = null;
			ISpecifyPropertyPages	spec = null;
			DsCAUUID cauuid = new DsCAUUID();

			try 
			{
				Guid cat  = PinCategory.Capture;
				Guid type = MediaType.Interleaved;
				Guid iid = typeof(IAMStreamConfig).GUID;
				hr = bld.FindInterface( ref cat, ref type, flt, ref iid, out comObj );
				if( hr != 0 )
				{
					type = MediaType.Video;
					hr = bld.FindInterface( ref cat, ref type, flt, ref iid, out comObj );
					if( hr != 0 )
						return false;
				}
				spec = comObj as ISpecifyPropertyPages;
				if( spec == null )
					return false;

				hr = spec.GetPages( out cauuid );
				hr = OleCreatePropertyFrame( hwnd, 30, 30, null, 1,
					ref comObj, cauuid.cElems, cauuid.pElems, 0, 0, IntPtr.Zero );
				return true;
			}
			catch( Exception ee )
			{
				Trace.WriteLine( "!Ds.NET: ShowCapPinDialog " + ee.Message );
				return false;
			}
			finally
			{
				if( cauuid.pElems != IntPtr.Zero )
					Marshal.FreeCoTaskMem( cauuid.pElems );
					
				spec = null;
				if( comObj != null )
					Marshal.ReleaseComObject( comObj ); comObj = null;
			}
		}

		public static bool ShowTunerPinDialog( ICaptureGraphBuilder2 bld, IBaseFilter flt, IntPtr hwnd )
		{
			int hr;
			object comObj = null;
			ISpecifyPropertyPages	spec = null;
			DsCAUUID cauuid = new DsCAUUID();

			try 
			{
				Guid cat  = PinCategory.Capture;
				Guid type = MediaType.Interleaved;
				Guid iid = typeof(IAMTVTuner).GUID;
				hr = bld.FindInterface( ref cat, ref type, flt, ref iid, out comObj );
				if( hr != 0 )
				{
					type = MediaType.Video;
					hr = bld.FindInterface( ref cat, ref type, flt, ref iid, out comObj );
					if( hr != 0 )
						return false;
				}
				spec = comObj as ISpecifyPropertyPages;
				if( spec == null )
					return false;

				hr = spec.GetPages( out cauuid );
				hr = OleCreatePropertyFrame( hwnd, 30, 30, null, 1,
					ref comObj, cauuid.cElems, cauuid.pElems, 0, 0, IntPtr.Zero );
				return true;
			}
			catch( Exception ee )
			{
				Trace.WriteLine( "!Ds.NET: ShowCapPinDialog " + ee.Message );
				return false;
			}
			finally
			{
				if( cauuid.pElems != IntPtr.Zero )
					Marshal.FreeCoTaskMem( cauuid.pElems );
					
				spec = null;
				if( comObj != null )
					Marshal.ReleaseComObject( comObj ); comObj = null;
			}
		}


		// from 'DShowUtil.cpp'
		public static int GetPin( IBaseFilter filter, PinDirection dirrequired, int num, out IPin ppPin )
		{
			ppPin = null;
			int hr;
			IEnumPins pinEnum;
			hr = filter.EnumPins( out pinEnum );
			if( (hr < 0) || (pinEnum == null) )
				return hr;

			IPin[] pins = new IPin[1];
			int f;
			PinDirection dir;
			do
			{
				hr = pinEnum.Next( 1, pins, out f );
				if( (hr != 0) || (pins[0] == null) )
					break;
				dir = (PinDirection) 3;
				hr = pins[0].QueryDirection( out dir );
				if( (hr == 0) && (dir == dirrequired) )
				{
					if( num == 0 )
					{
						ppPin = pins[0];
						pins[0] = null;
						break;
					}
					num--;
				}
				Marshal.ReleaseComObject( pins[0] ); pins[0] = null;
			}
			while( hr == 0 );

			Marshal.ReleaseComObject( pinEnum ); pinEnum = null;
			return hr;
		}

		public static int GetPins( IBaseFilter filter, PinDirection dirrequired, out ArrayList pins )
		{
			pins = null;
			int hr;
			IEnumPins pinEnum;
			hr = filter.EnumPins( out pinEnum );
			if( (hr < 0) || (pinEnum == null) )
				return hr;

			IPin[] apin = new IPin[1];
			int f;
			PinDirection dir;
			do
			{
				hr = pinEnum.Next( 1, apin, out f );
				if( (hr != 0) || (apin[0] == null) )
					break;
				dir = (PinDirection) 3;
				hr = apin[0].QueryDirection( out dir );
				if( (hr == 0) && (dir == dirrequired) )
				{
					DsPin pin = new DsPin();
					PinInfo info = new PinInfo();
					apin[0].QueryPinInfo(info);
					if( info.pFilter != null )
						Marshal.ReleaseComObject(info.pFilter);
					pin.Name = info.achName;

					IKsPropertySet pKs = apin[0] as IKsPropertySet;
					if (pKs != null) 
					{
						Guid propSet = AMPropSetID.Pin;
						Guid pinCategory = new Guid();
						hr = pKs.Get(ref propSet, AMProperty_Pin.Category, 0, 0, out pinCategory, Marshal.SizeOf(pinCategory), out f);
						pin.Category = pinCategory;
					}

					if( pins == null )
						pins = new ArrayList();
					pin.Pin = apin[0]; apin[0] = null;
					pins.Add(pin); pin = null;

				}
				//Marshal.ReleaseComObject( apin[0] ); 
				apin[0] = null;
			}
			while( hr == 0 );

			Marshal.ReleaseComObject( pinEnum ); pinEnum = null;
			return hr;
		}

		public static int FindPin( IBaseFilter filter, PinDirection dirrequired, string name, out IPin ppPin )
		{
			ppPin = null;
			int hr;
			IEnumPins pinEnum;
			hr = filter.EnumPins( out pinEnum );
			if( (hr < 0) || (pinEnum == null) )
				return hr;

			IPin[] pins = new IPin[1];
			int f;
			PinDirection dir;
			do
			{
				hr = pinEnum.Next( 1, pins, out f );
				if( (hr != 0) || (pins[0] == null) )
					break;
				dir = (PinDirection) 3;
				hr = pins[0].QueryDirection( out dir );
				if( (hr == 0) && (dir == dirrequired) )
				{
					PinInfo info = new PinInfo();
					pins[0].QueryPinInfo(info);
					if( info.pFilter != null )
						Marshal.ReleaseComObject(info.pFilter);

					if( name == info.achName )
					{
						ppPin = pins[0];
						pins[0] = null;
						break;
					}
				}
				Marshal.ReleaseComObject( pins[0] ); pins[0] = null;
			}
			while( hr == 0 );

			Marshal.ReleaseComObject( pinEnum ); pinEnum = null;
			return hr;
		}

		public static int GetMediaTypes( IPin pin, out ArrayList mediatypes )
		{
			mediatypes = null;
			int hr;
			IEnumMediaTypes mediaTypeEnum;
			hr = pin.EnumMediaTypes( out mediaTypeEnum );
			if( (hr < 0) || (mediaTypeEnum == null) )
				return hr;

			AMMediaType[] amediatype = new AMMediaType[1];
			int f;
			do
			{
				hr = mediaTypeEnum.Next( 1, out amediatype, out f );
				if( (hr != 0) )
					break;
				DsAMMediaType mediaType = new DsAMMediaType();
				mediaType.MediaType = amediatype[0];

				if (amediatype[0].formatType == FormatType.VideoInfo)
				{
					mediaType.videoInfoHeader = (VideoInfoHeader)Marshal.PtrToStructure(amediatype[0].formatPtr, typeof(VideoInfoHeader));
				}
				
				if( mediatypes == null )
					mediatypes = new ArrayList();
				mediatypes.Add(mediaType);

				// DeleteMediaType(amediatype[0]);
				if (amediatype[0].formatSize != 0)
					Marshal.FreeCoTaskMem(amediatype[0].formatPtr);
				if (amediatype[0].unkPtr != IntPtr.Zero)
					Marshal.FreeCoTaskMem(amediatype[0].unkPtr);
				//Marshal.FreeCoTaskMem(amediatype[0]);
			}
			while( hr == 0 );

			Marshal.ReleaseComObject( mediaTypeEnum ); mediaTypeEnum = null;
			return hr;
		}

		public static int GetFormat( IAMStreamConfig pin, out DsAMMediaType mediatype )
		{
			mediatype = null;
			int hr;

			AMMediaType[] amediatype = new AMMediaType[1];
			IntPtr pmt;
			hr = pin.GetFormat(out pmt);
			if( (hr != 0) )
				return hr;

			amediatype[0] = (AMMediaType)Marshal.PtrToStructure(pmt, typeof(AMMediaType));

			mediatype = new DsAMMediaType();
			mediatype.MediaType = amediatype[0];

			if (amediatype[0].formatType == FormatType.VideoInfo)
			{
				mediatype.videoInfoHeader = (VideoInfoHeader)Marshal.PtrToStructure(amediatype[0].formatPtr, typeof(VideoInfoHeader));
			}

			// DeleteMediaType(amediatype[0]);
			if (amediatype[0].formatSize != 0)
				Marshal.FreeCoTaskMem(amediatype[0].formatPtr);
			if (amediatype[0].unkPtr != IntPtr.Zero)
				Marshal.FreeCoTaskMem(amediatype[0].unkPtr);
			if (pmt != IntPtr.Zero)
				Marshal.FreeCoTaskMem(pmt);

			return hr;
		}

		public static int SetFormat( IAMStreamConfig pin, DsAMMediaType mediatype )
		{
			int hr=0;

			AMMediaType[] amediatype = new AMMediaType[1];
			IntPtr pmt;
			hr = pin.GetFormat(out pmt);
			if( (hr != 0) )
				return hr;
			
			amediatype[0] = (AMMediaType)Marshal.PtrToStructure(pmt, typeof(AMMediaType));

			try
			{
				amediatype[0].majorType = mediatype.MediaType.majorType;
				amediatype[0].subType = mediatype.MediaType.subType;
				amediatype[0].fixedSizeSamples = mediatype.MediaType.fixedSizeSamples;
				amediatype[0].temporalCompression = mediatype.MediaType.temporalCompression;
				amediatype[0].sampleSize = mediatype.MediaType.sampleSize;

				if (amediatype[0].formatType == FormatType.VideoInfo)
				{
					Marshal.StructureToPtr(mediatype.videoInfoHeader, amediatype[0].formatPtr, true);
				}

				Marshal.StructureToPtr(amediatype[0], pmt, false);

				hr = pin.SetFormat(pmt);
				if( (hr != 0) )
					return hr;
			}
			finally
			{
				// DeleteMediaType(amediatype[0]);
				if ((amediatype != null) && (amediatype[0].formatSize != 0))
					Marshal.FreeCoTaskMem(amediatype[0].formatPtr);
				if ((amediatype != null) && (amediatype[0].unkPtr != IntPtr.Zero))
					Marshal.FreeCoTaskMem(amediatype[0].unkPtr);
				if (pmt != IntPtr.Zero)
					Marshal.FreeCoTaskMem(pmt);
			}

			return hr;
		}

		public static int GetStreamCaps( IAMStreamConfig pin, out ArrayList mediatypes )
		{
			mediatypes = null;
			int hr;
			int iCount;
			int iSize;
			hr = pin.GetNumberOfCapabilities(out iCount, out iSize);
			if( (hr < 0) )
				return hr;

			AMMediaType[] amediatype = new AMMediaType[1];
			IntPtr f = Marshal.AllocHGlobal(iSize);
			int iMarshalSize = Marshal.SizeOf(typeof(VIDEO_STREAM_CONFIG_CAPS));
			for (int i = 0; i < iCount; i++)
			{
				hr = pin.GetStreamCaps( i, out amediatype, f );
				if( (hr != 0) )
					break;
				DsAMMediaType mediaType = new DsAMMediaType();
				mediaType.MediaType = amediatype[0];

				if (amediatype[0].formatType == FormatType.VideoInfo)
				{
					mediaType.videoInfoHeader = (VideoInfoHeader)Marshal.PtrToStructure(amediatype[0].formatPtr, typeof(VideoInfoHeader));
					mediaType.videoStreamConfigCaps = (VIDEO_STREAM_CONFIG_CAPS)Marshal.PtrToStructure(f, typeof(VIDEO_STREAM_CONFIG_CAPS));
				}
				
				if( mediatypes == null )
					mediatypes = new ArrayList();
				mediatypes.Add(mediaType);

				// DeleteMediaType(amediatype[0]);
				if (amediatype[0].formatSize != 0)
					Marshal.FreeCoTaskMem(amediatype[0].formatPtr);
				if (amediatype[0].unkPtr != IntPtr.Zero)
					Marshal.FreeCoTaskMem(amediatype[0].unkPtr);
			}

			Marshal.FreeHGlobal(f);
			return hr;
		}

		[ComVisible(false)]
			public class DsPin : IDisposable
		{
			public string			Name;
			public IPin				Pin;
			public Guid				Category;
		
			public void Dispose()
			{
				if( Pin != null )
					Marshal.ReleaseComObject( Pin ); Pin = null;
			}

			public override string ToString()
			{
				return Name;
			}
		}


		[ComVisible(false)]
			public class DsAMMediaType : IDisposable
		{
			public AMMediaType		MediaType;
			public VideoInfoHeader	videoInfoHeader;
			public VIDEO_STREAM_CONFIG_CAPS videoStreamConfigCaps;
		
			public void Dispose()
			{
				Marshal.ReleaseComObject( MediaType );
			}

		}


		[DllImport("olepro32.dll", CharSet=CharSet.Unicode, ExactSpelling=true) ]
		private static extern int OleCreatePropertyFrame( IntPtr hwndOwner, int x, int y,
			string lpszCaption, int cObjects,
			[In, MarshalAs(UnmanagedType.Interface)] ref object ppUnk,
			int cPages,	IntPtr pPageClsID, int lcid, int dwReserved, IntPtr pvReserved );
	}


	// ---------------------------------------------------------------------------------------

	[StructLayout(LayoutKind.Sequential), ComVisible(false)]
	public struct DsPOINT		// POINT
	{
		public int		X;
		public int		Y;
	}


	// ---------------------------------------------------------------------------------------

	[StructLayout(LayoutKind.Sequential), ComVisible(false)]
	public struct DsRECT		// RECT
	{
		public int		Left;
		public int		Top;
		public int		Right;
		public int		Bottom;
	}


	// ---------------------------------------------------------------------------------------

	[StructLayout(LayoutKind.Sequential, Pack=2), ComVisible(false)]
	public struct DsBITMAPINFOHEADER
	{
		public int		Size;
		public int		Width;
		public int		Height;
		public short	Planes;
		public short	BitCount;
		public int		Compression;
		public int		ImageSize;
		public int		XPelsPerMeter;
		public int		YPelsPerMeter;
		public int		ClrUsed;
		public int		ClrImportant;
	}


	// ---------------------------------------------------------------------------------------

	[StructLayout(LayoutKind.Sequential), ComVisible(false)]
	public struct VIDEO_STREAM_CONFIG_CAPS
	{
		public Guid		guid;				// will be MEDIATYPE_Video
		public int		VideoStandard;		// logical OR of all AnalogVideoStandards
		 									// supported
		public DsPOINT	InputSize;			// the inherent size of the incoming signal
		 									// (every pixel unique)
		public DsPOINT	MinCroppingSize;	// smallest rcSrc cropping rect allowed
		public DsPOINT	MaxCroppingSize;	// largest rcSrc cropping rect allowed
		public int		CropGranularityX;	// granularity of cropping size
		public int		CropGranularityY;
		public int		CropAlignX;			// alignment of cropping rect
		public int		CropAlignY;
		public DsPOINT	MinOutputSize;		// smallest bitmap stream can produce
		public DsPOINT	MaxOutputSize;		// largest  bitmap stream can produce
		public int		OutputGranularityX;	// granularity of output bitmap size
		public int		OutputGranularityY;
		public int		StretchTapsX;		// 0, no stretch, 1 pix dup, 2 interp, ...
		public int		StretchTapsY;		//    Describes quality of hardware scaler
		public int		ShrinkTapsX;		//
		public int		ShrinkTapsY;		//
		public long		MinFrameInterval;	// 100 nS units
		public long		MaxFrameInterval;
		public int		MinBitsPerSecond;
		public int		MaxBitsPerSecond;
	}




	// ---------------------------------------------------------------------------------------

	[ComVisible(false)]
	public class DsROT
	{
		public static bool AddGraphToRot( object graph, out int cookie )
		{
			cookie = 0;
			int hr = 0;
			UCOMIRunningObjectTable rot = null;
			UCOMIMoniker mk = null;
			try 
			{
				hr = GetRunningObjectTable( 0, out rot );
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );

				int id = GetCurrentProcessId();
				IntPtr iuPtr = Marshal.GetIUnknownForObject( graph );
				int iuInt = (int) iuPtr;
				Marshal.Release( iuPtr );
				string item = string.Format( "FilterGraph {0} pid {1}", iuInt.ToString("x8"), id.ToString("x8") );
				hr = CreateItemMoniker( "!", item, out mk );
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );
				
				rot.Register( ROTFLAGS_REGISTRATIONKEEPSALIVE, graph, mk, out cookie );
				return true;
			}
			catch( Exception )
			{
				return false;
			}
			finally
			{
				if( mk != null )
					Marshal.ReleaseComObject( mk ); mk = null;
				if( rot != null )
					Marshal.ReleaseComObject( rot ); rot = null;
			}
		}

		public static bool RemoveGraphFromRot( ref int cookie )
		{
			UCOMIRunningObjectTable rot = null;
			try 
			{
				int hr = GetRunningObjectTable( 0, out rot );
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );

				rot.Revoke( cookie );
				cookie = 0;
				return true;
			}
			catch( Exception )
			{
				return false;
			}
			finally
			{
				if( rot != null )
					Marshal.ReleaseComObject( rot ); rot = null;
			}
		}

		private const int ROTFLAGS_REGISTRATIONKEEPSALIVE	= 1;

		[DllImport("ole32.dll", ExactSpelling=true) ]
		private static extern int GetRunningObjectTable( int r,
			out UCOMIRunningObjectTable pprot );

		[DllImport("ole32.dll", CharSet=CharSet.Unicode, ExactSpelling=true) ]
		private static extern int CreateItemMoniker( string delim,
			string item, out UCOMIMoniker ppmk );

		[DllImport("kernel32.dll", ExactSpelling=true) ]
		private static extern int GetCurrentProcessId();
	}





	// ---------------------------------- ocidl.idl ------------------------------------------------

	[ComVisible(true), ComImport,
	Guid("B196B28B-BAB4-101A-B69C-00AA00341D07"),
	InterfaceType( ComInterfaceType.InterfaceIsIUnknown )]
	public interface ISpecifyPropertyPages
	{
		[PreserveSig]
		int GetPages( out DsCAUUID pPages );
	}

	[StructLayout(LayoutKind.Sequential), ComVisible(false)]
	public struct DsCAUUID		// CAUUID
	{
		public int		cElems;
		public IntPtr	pElems;
	}

	// ---------------------------------------------------------------------------------------


	[StructLayout(LayoutKind.Sequential), ComVisible(false)]
	public class DsOptInt64
	{
		public DsOptInt64( long Value )
		{
			this.Value = Value;
		}
		public long		Value;
	}


	[StructLayout(LayoutKind.Sequential), ComVisible(false)]
	public class DsOptIntPtr
	{
		public IntPtr	Pointer;
	}



} // namespace DShowNET
